/*
 * Test program for syntax.c: reads a file on stdin or from argv[1] and
 * uses ANSI (not curses or termcap) to print it in pretty colors.
 *
 * Copyright (C) 1998,1999,2004 by Patrick Reynolds <reynolds@cs.duke.edu>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */

#include "syntax.h"
#include <stdio.h>
#include <string.h>
#include <unistd.h>

/* order matters here.  See the enum in syntax.h */
const char *highlight[] = {
  "\033[1;33m",    /* yellow */
  "\033[1;37m",    /* white */
  "\033[0;37m",    /* grey */
  "\033[1;36m",    /* b-cyan */
  "\033[1;36m",    /* b-cyan */
  "\033[1;32m",    /* b-green */
  "\033[0;33m",    /* brown */
  "\033[0;36m",    /* plain cyan */
  "\033[1;31m",    /* b-red */
  "\033[0;37m"     /* whitespace */
};

#define TABLEN 8

int main(int argc, char **argv) {
  int i, di, line=1;
  char space = 183, tab[] = { 187, 32, 32, 32, 32, 32, 32, 32 };
  syntax_t cur = UNDEFINED;
  FILE *fin;
  char buf[4096];  /* there had better not be any lines longer than this! */

  language = JAVA;
  write(1, highlight[WHITESPACE], strlen(highlight[WHITESPACE]));

  if (argc > 2) {
    printf("Usage:\n  syntax foo.c\n\n");
    return -1;
  }

  if (argc == 2) {
    if (!(fin = fopen(argv[1], "r"))) {
      perror("fopen");
      return -1;
    }
  }
  else fin=stdin;

  while (fgets(buf, 4096, fin)) {
    int pos = 0;
    printf("\033[0m%5d: ", line++); fflush(stdout);
    for (i=0; buf[i]; ) {
      if (buf[i] != '\n' && buf[i] != '\r') {
        syntax_t syn = get_syntax(buf, i, &di, !i);
        if (syn != cur) {
          write(1, highlight[syn], strlen(highlight[syn]));
          cur = syn;
        }
      }
      else {
        di = 1;
        cur = UNDEFINED;
      }
      if (cur == WHITESPACE) {
        int j;
        for (j=0; j<di; j++)
          if (buf[i+j] == ' ') {
            write(1, &space, 1);
            pos++;
          }
          else if (buf[i] == '\t') {
            int spaces = TABLEN - (pos % TABLEN);
            write(1, tab, spaces);
            pos += spaces;
          }
      }
      else {
        write(1, &buf[i], di);
        pos += di;
      }
      i += di;
    }
  }

  write(1, "\033[0m", 4);

  return 0;
}
